@@ -13,7 +13,7 @@
 
 package IO::Socket::SSL;
 
-our $VERSION = '2.033';
+our $VERSION = '2.034';
 
 use IO::Socket;
 use Net::SSLeay 1.46;
@@ -580,33 +580,11 @@ sub configure_SSL {
 
     # add user defined defaults, maybe after filtering
     $FILTER_SSL_ARGS->($is_server,$arg_hash) if $FILTER_SSL_ARGS;
-    my %defaults = (
-	%$GLOBAL_SSL_ARGS,
-	$is_server ? %$GLOBAL_SSL_SERVER_ARGS : %$GLOBAL_SSL_CLIENT_ARGS,
-    );
-    if ( $defaults{SSL_reuse_ctx} ) {
-	# ignore default context if there are args to override it
-	delete $defaults{SSL_reuse_ctx}
-	    if grep { m{^SSL_(?!verifycn_name|hostname)$} } keys %$arg_hash;
-    }
-    %$arg_hash = ( %defaults, %$arg_hash ) if %defaults;
 
-    my $ctx = $arg_hash->{'SSL_reuse_ctx'};
-    if ($ctx) {
-	if ($ctx->isa('IO::Socket::SSL::SSL_Context') and
-	    $ctx->{context}) {
-	    # valid context
-	} elsif ( $ctx = ${*$ctx}{_SSL_ctx} ) {
-	    # reuse context from existing SSL object
-	}
-    }
-
-    # create context
-    # this will fill in defaults in $arg_hash
-    $ctx ||= IO::Socket::SSL::SSL_Context->new($arg_hash) || return;
+    # this adds defaults to $arg_hash as a side effect!
+    ${*$self}{'_SSL_ctx'} = IO::Socket::SSL::SSL_Context->new($arg_hash);
 
     ${*$self}{'_SSL_arguments'} = $arg_hash;
-    ${*$self}{'_SSL_ctx'} = $ctx;
     ${*$self}{'_SSL_opened'} = 1 if $is_server;
 
     return $self;
@@ -2148,10 +2126,33 @@ sub new {
     #DEBUG( "$class @_" );
     my $arg_hash = (ref($_[0]) eq 'HASH') ? $_[0] : {@_};
 
+    my $is_server = $arg_hash->{SSL_server};
+    my %defaults = (
+	%$GLOBAL_SSL_ARGS,
+	$is_server 
+	    ? (%$GLOBAL_SSL_SERVER_ARGS, %DEFAULT_SSL_SERVER_ARGS) 
+	    : (%$GLOBAL_SSL_CLIENT_ARGS, %DEFAULT_SSL_CLIENT_ARGS),
+    );
+    if ( $defaults{SSL_reuse_ctx} ) {
+	# ignore default context if there are args to override it
+	delete $defaults{SSL_reuse_ctx}
+	    if grep { m{^SSL_(?!verifycn_name|hostname)$} } keys %$arg_hash;
+    }
+    %$arg_hash = ( %defaults, %$arg_hash ) if %defaults;
+
+    if (my $ctx = $arg_hash->{'SSL_reuse_ctx'}) {
+	if ($ctx->isa('IO::Socket::SSL::SSL_Context') and
+	    $ctx->{context}) {
+	    # valid context
+	} elsif ( $ctx = ${*$ctx}{_SSL_ctx} ) {
+	    # reuse context from existing SSL object
+	}
+	return $ctx
+    }
+
     # common problem forgetting to set SSL_use_cert
     # if client cert is given by user but SSL_use_cert is undef, assume that it
     # should be set
-    my $is_server = $arg_hash->{SSL_server};
     if ( ! $is_server && ! defined $arg_hash->{SSL_use_cert}
 	&& ( grep { $arg_hash->{$_} } qw(SSL_cert SSL_cert_file))
 	&& ( grep { $arg_hash->{$_} } qw(SSL_key SSL_key_file)) ) {
@@ -2170,11 +2171,8 @@ sub new {
     }
 
     # add library defaults
-    %$arg_hash = (
-	SSL_use_cert => $is_server,
-	$is_server ? %DEFAULT_SSL_SERVER_ARGS : %DEFAULT_SSL_CLIENT_ARGS,
-	%$arg_hash
-    );
+    $arg_hash->{SSL_use_cert} = $is_server if ! defined $arg_hash->{SSL_use_cert};
+
 
     # Avoid passing undef arguments to Net::SSLeay
     defined($arg_hash->{$_}) or delete($arg_hash->{$_}) for(keys %$arg_hash);
